/**
 * @author 503722728@qq.com
 * drawcall 优化组件能轻松优化list相关label 图集相互打断导致的draw call很高
 * 如果因为使用对象池导致出现奇怪的bug 请在对象池取出对象时 手动重置一次item renderFlag
 * this.node._renderFlag = cc.RenderFlow.FLAG_CHILDREN
 */

const { ccclass, property, menu, executionOrder, help } = cc._decorator;

let containerZIndex: number = 1;

@ccclass("OptimizeDrawCallOptions")
export class OptimizeDrawCallOptions {

    /**
     * 待优化节点的路径，层级使用斜线分隔
     * 例如：nodeNameA/nodeNameB/nodeNameC
     */
    @property({
        type: cc.String,
        displayName: '待优化节点名',
        tooltip: '脚本节点下的节点名 优化节点不能重名'
    })
    pathToNode: string[] = [""];

    /**
     * 记录上次的数组长度
     */
    oldLength = 1;

    /**
     * 分层节点的zIndex
     */
    @property({
        type: cc.Integer,
        displayName: '分层节点zIndex',
        tooltip: '如层级不对，可自行调整，一般节点层级默认为0'
    })
    containerZIndex: number = containerZIndex;
}

@ccclass
@menu("优化组件/DrawCall")
@executionOrder(1)
export default class OptimizeDrawCall extends cc.Component {
    @property({
        displayName: '开启优化',
        tooltip: '用于对比优化效果'
    })
    enabledOptimize: boolean = true

    @property()
    _options: OptimizeDrawCallOptions[] = [new OptimizeDrawCallOptions()]

    @property({
        type: OptimizeDrawCallOptions,
        displayName: '优化项',
        tooltip: '不同层级'
    })
    get options() {
        return this._options
    }

    set options(value) {
        if (CC_EDITOR) {
            value.forEach((op, i) => {
                let arr = op.pathToNode
                let last = arr[arr.length - 1]
                
                // cc.error(JSON.stringify(value))
                if (arr.length < op.oldLength) {
                    arr[arr.length - 1] = ""
                } else if (last != "") {
                    arr.push("")
                }
                op.oldLength = arr.length
            })
            containerZIndex = value.length + 1
        }

        this._options = value
        
    }

    start() {
        if (!cc.sys.isNative)
            this._setNodeRender()
    }

    _setNodeRender() {
        if (!this.enabledOptimize) {
            // 是否开启优化
            return
        }

        let config = {

        }

        this.options.forEach(nodeInfo => {
            if (nodeInfo) nodeInfo.pathToNode.forEach(name => {
                // 添加特殊配置
                if (name) config[name] = nodeInfo.containerZIndex
            })
        })

        let content = this.node;
        content._renderFlag &= ~cc.RenderFlow.FLAG_CHILDREN;
        content._renderFlag |= cc.RenderFlow.FLAG_CHILDREN_BATCH_RENDER;
        content.config = config;
    }
}

